#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-hydra-config v. 1.0.0
#
# Post-installation script for configuring the gLite data hydra service
#
# Joachim Flammer <Joachim.Flammer@cern.ch>
#
# Version info: $Id: glite-hydra-config.py,v 1.2.2.3 2005/12/08 13:10:09 flammer Exp $
#
# Usage: python glite-hydra-config [-c|-v|-h|--start|--stop|--status|--configure|--help]
#        -c          print configuration
#        -v          print version
#        -h,--help   print usage info
#        --verbose   turn verbosity on
#
#        --start     start the services
#        --stop      stop the services
#        --status    show the status of the services
#        --configure configure the services
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################

import os,string,socket,pwd
import sys
import getopt
import commands
import time

from gLiteInstallerLib import gLib
from gLiteInstallerLib import ConfigParams

from gliteRgmaServicetool import gliteRgmaServicetool
from gliteRgmaServicetool import gliteRgmaServicetoolInstance

from mysql import Mysql
from tomcat5 import Secure_tomcat5 as Tomcat5

# Set global variables here 
global params                        # all config values from the XML file
global mysql                         # the one and only mysql service instance
global verbose
global rgmaServicetool

class glite_hydra:

    #################################################################
    # init 
    #
    # init service class
    #
    #################################################################
    def __init__(self,params):
        
        self.verbose = 0
        self.version = "1.0.0"
        self.name = "glite-hydra"
        self.friendly_name = "gLite Hydra"
        
        self.tomcat = Tomcat5(params)
        self.mysql = Mysql()
        
        self.params = params
        params['module.version'] = self.version
        
        self.serviceType_hydra = "org.glite.MetadataCatalog"
        
        self.serviceStatusScriptBase = "%s/etc/config/scripts/gLiteCheckWebserviceStatus.py" % os.environ['GLITE_LOCATION']        
        self.serviceStatusMethod     = "/"        
        
        self.serviceName = "gLite encrypted data storage server for VO "        

        # endpoint is base/voName
        self.serviceEndpointBase = "https://%s:8443" % os.environ['HOSTNAME']
        
        # wsdl is base1/voName/base2
        self.serviceWsdlUrlBase1 = "https://%s:8443" % os.environ['HOSTNAME']
        self.serviceWsdlUrlBase2 = "/services/Hydra?wsdl"
        
    #################################################################
    # banner 
    #
    # show banner
    #
    #################################################################
    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "    Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"

    
    #################################################################
    # copyright 
    #
    # show copyright
    #
    #################################################################
    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'


    #################################################################
    # showVersion 
    #
    # show version
    #
    #################################################################
    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    

    #################################################################
    # usage 
    #
    # print usage
    #
    #################################################################
    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print "Usage: "
        print "Edit the configuration file %s.cfg.xml in %s/etc.config/templates" % (self.name, os.environ['GLITE_LOCATION'])
        print "save it as %s/etc/config/%s.cfg.xml and run the script as follows" % (os.environ['GLITE_LOCATION'], self.name)
        print "python %s-config [OPTION...]\n" % self.name
        print '    -c, --checkconf     print the service configuration'
        print '    -v, --version       print the version of the configuration script'
        print '    -h, --help          print this usage information'
        print '    --verbose           turn verbosity on'
        print
        print '    --start             start the %s' % self.friendly_name
        print '    --stop              stop the %s' % self.friendly_name
        print '    --status            print service status (0=running, 1=idle)'
        print '    --configure         configure the service'
        print '    --siteconfig=URL    URL of the site configuration file'
        print '\n'


    #################################################################
    # configure
    #
    # configure rgma server
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configure(self):
        
        # ------------------------------------------------------------        
        # Check certificates
        # ------------------------------------------------------------    
        if params.has_key('glite.installer.checkcerts'):
            if params['glite.installer.checkcerts'] == "true":
                if glib.check_certs(params) != 0:
                    print "An error occurred while configuring the %s service" % service.friendly_name
                    sys.exit(1)

        # ------------------------------------------------------------
        # Run security util script
        # ------------------------------------------------------------
                
        print "Configuring gLite security ...                                 "
        errorcode = os.system("%s/etc/config/scripts/glite-security-utils-config.py --subservice" % os.environ['GLITE_LOCATION'])

        if errorcode:
            print "Configuring gLite security                                ",
            glib.printFailedMessage()
            return 1
                    
        # ------------------------------------------------------------
        # get non vo specific configuration values
        # ------------------------------------------------------------
                
        print "Checking configuration values                             ",
        
        # mysql root password (user)
        if params.has_key('hydra.mysql.admin.password'):
            self.mysql_root_password = params['hydra.mysql.admin.password']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"hydra.mysql.admin.password\" not found.")
            return 1

        # mysql admin password => to create users (advanced)
        self.setMysqlAdminPassword = params['set.mysql.root.password']        
        
        # db schema file (system)
        self.db_schema_file = params['hydra.schemaFile']
        
        # activate rgma servicetool (advanced)
        self.rgma_servicetool_activate = params['rgma.servicetool.activate']
        if (self.rgma_servicetool_activate != "true") and (self.rgma_servicetool_activate != "false"):
            glib.printFailedMessage()
            glib.printErrorMessage("Incorrect parameter 'rgma.servicetool.activate' given.")
            glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.rgma_servicetool_activate)
            return 1
        
        glib.printOkMessage()
        
        #--------------------------------------------------------
        # Get R-GMA settings
        #--------------------------------------------------------
        if self.rgma_servicetool_activate=="true":
     
            # we create the R_GMA user here - this is normally done by the 
            # R-GMA servicetool but we need the home directory of it already
            if rgmaServicetool.configureRgmaUser(glib):
                return 1

            # rgma user name (system)
            rgmaUserName = params['rgma.user.name']

            # certificate path (system)
            userCertificatePath = params['user.certificate.path']

            # derive rgma user key and certificate path
            rgmaUserHomeDir     = pwd.getpwnam(rgmaUserName)[5] 
            self.rgmaCertificatePath = "%s/%s/hostcert.pem" % (rgmaUserHomeDir, userCertificatePath)
            self.rgmaKeyPath         = "%s/%s/hostkey.pem"  % (rgmaUserHomeDir, userCertificatePath)
         
        #-------------------------------------------------------------------------------
        # Verify the JAVA installation and set it in /etc/java.conf
        #-------------------------------------------------------------------------------
        
        if glib.checkJava() != 0:
            return 1
        
        # ------------------------------------------------------------
        # Configure MySQL
        # ------------------------------------------------------------
        
        print "\nConfiguring MySQL ...                                    "        
                           
        # start MySQL
        errorcode = mysql.start()
        if errorcode:
            return 1
        
        if mysql.checkMySQLConfiguration(self.mysql_root_password,self.setMysqlAdminPassword):
            print "Configuring MySQL                                         ",
            glib.printFailedMessage()
            return 1

        print "Configuring MySQL                                         ",
        glib.printOkMessage()
                        
        # ------------------------------------------------------------
        # Configure tomcat        
        # ------------------------------------------------------------        
        print "\nConfiguring Tomcat ...                                  "
        
        # stop tomcat first
        tomcat.stop()
        
        # configure tomcat
        errorcode = tomcat.configure()
        if errorcode != 0:
            print "Configuring Tomcat                                            ",
            glib.printFailedMessage()
            return 1
             
        print "Configuring Tomcat                                        ",
        glib.printOkMessage()

        # ------------------------------------------------------------
        #  Configure VO instances
        # ------------------------------------------------------------
        instances = glib.getInstances('hydra')
        if len(instances) == 0:
            errorcode = self.configureInstance()
            if errorcode:
                return 1
        else:
            for name in instances.keys():
                errorcode = self.configureInstance(name)
                if errorcode:
                    return 1
        
        #--------------------------------------------------------
        # RGMA servicetool: configure servicetool
        #--------------------------------------------------------        
        print
        if self.rgma_servicetool_activate=="true":
            if rgmaServicetool.configure(glib):
                return 1        
        
        # ------------------------------------------------------------
        # registering HYDRA server
        # ------------------------------------------------------------
        
        glib.registerService()
            
        return 0


    #################################################################
    # configureInstance
    #
    # configure one hydra service vo instance
    #
    # params instanceName  name of instance (normally vo name)
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configureInstance(self, instanceName='default:default'):

        # extract vo name from 'default:voName' string
        voNameArray = instanceName.split(":")
        default_vo_name = voNameArray[1]

        print "\nConfiguring VO '%s' ..." % default_vo_name        

        #--------------------------------------------------------
        # Get and check configuration parameters
        #--------------------------------------------------------
        
        print "Checking VO specific configuration values                 ",
         
        try:
            instanceParams = glib.getInstanceParams(instanceName)
        except ValueError, msg:
            glib.printFailedMessage()
            glib.printErrorMessage("%s, Cannot continue" % msg)
            return 1
        except LookupError, msg:
            glib.printFailedMessage()
            glib.printErrorMessage(msg)
            return 1

        #--------------------------------------------------------
        # hydra vo specific configuration values
        #--------------------------------------------------------

        # vo name - if not present: assign name of instance (user)
        if instanceParams.has_key('vo.name'):
            self.vo_name = instanceParams['vo.name']
        else:
            self.vo_name = default_vo_name
                        
        # vo name (user)
        if instanceParams.has_key('hydra.DBNAME'):
            self.db_name = instanceParams['hydra.DBNAME']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"hydra.DBNAME\" not found.")
            return 1

        # db name (user)
        if instanceParams.has_key('hydra.DBNAME'):
            self.db_name = instanceParams['hydra.DBNAME']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"hydra.DBNAME\" not found.")
            return 1
            
        # db user name (user)
        if instanceParams.has_key('hydra.DBUSER'):
            self.db_user_name = instanceParams['hydra.DBUSER']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"hydra.DBUSER\" not found.")
            return 1
            
        # db password (user)
        if instanceParams.has_key('hydra.DBPASSWORD'):
            self.db_password = instanceParams['hydra.DBPASSWORD']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"hydra.DBPASSWORD\" not found.")
            return 1
        
        # webapp path (system)                            
        if instanceParams.has_key('hydra.PATH'):
            self.webappPath = instanceParams['hydra.PATH']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"hydra.PATH\" not found.")
            return 1 
               
        glib.printOkMessage()
        
        # ------------------------------------------------------------
        #    Create DB user
        # ------------------------------------------------------------
                
        if mysql.add_userWithPwd(self.db_name, self.db_user_name, self.db_password, self.mysql_root_password):
            glib.printFailedMessage()
            return 1
            
        # ------------------------------------------------------------
        #    Load schema
        # ------------------------------------------------------------
        print "Loading DB schema into MySQL                              ",
        
        errorcode = os.system("mysql -u%s -p%s %s < %s" \
            % (self.db_user_name, self.db_password, self.db_name, self.db_schema_file))
        if errorcode:
            glib.printFailedMessage()
            return 1

        glib.printOkMessage()

        # ------------------------------------------------------------
        #    Replace tokens in the context.xml template file
        # ------------------------------------------------------------
        
        if self.verbose:
            print "Configuring context.xml ...                                   "
        
        inFile = open("%s/etc/glite-data-hydra-service/context.xml.template" % os.environ['GLITE_LOCATION'],"r")
        outFile = open("%s/conf/Catalina/localhost/glite-data-hydra-service.%s.xml" % \
            (os.environ['CATALINA_HOME'], self.vo_name) ,"w")

        for line in inFile.readlines():
            line = self.replaceTags(line, instanceParams, 'hydra')
            outFile.write(line)

        outFile.close()
        inFile.close()
        
        print "Configuring context.xml                                   ",
        glib.printOkMessage()

        # ------------------------------------------------------------
        #  Copy jar file (should this be changed?) - is tomcat location correct?
        # ------------------------------------------------------------      
        print "Installing web service                                    ",
        errorcode = os.system("cp %s/lib/glite-data-common/*.jar /usr/share/tomcat5/common/lib/" \
            % os.environ['GLITE_LOCATION'])   
        if errorcode:
            glib.printFailedMessage()
            return 1

        glib.printOkMessage()

        #--------------------------------------------------------
        # Configure R-GMA servicetool instance
        #--------------------------------------------------------                        

        if self.rgma_servicetool_activate=="true":
        
            # Create one instance
            servicetoolInstance = gliteRgmaServicetoolInstance(glib, "%s_%s_%s" % (os.environ['HOSTNAME'], self.vo_name, self.serviceType_hydra))
        
            # set servicetool instance params
            servicetoolInstance.setServiceType(self.serviceType_hydra)
            servicetoolInstance.setServiceVersion(self.version)
            servicetoolInstance.setStatusScript("%s --endpoint %s%s --key %s --certificate %s" % 
                (self.serviceStatusScriptBase, self.webappPath, self.serviceStatusMethod, self.rgmaKeyPath, self.rgmaCertificatePath))
            servicetoolInstance.setEndpoint("%s%s"     % (self.serviceEndpointBase, self.webappPath))
            servicetoolInstance.setServiceName("%s%s"  % (self.serviceName, self.vo_name))
            servicetoolInstance.setWsdlUrl("%s%s%s"    % (self.serviceWsdlUrlBase1, self.webappPath, self.serviceWsdlUrlBase2))
            servicetoolInstance.add() 

        print "Configuring VO '%s'                                     " % self.vo_name,
        glib.printOkMessage()
        
        return 0

    
    #################################################################
    # replaceTags
    #
    # replace data tags in context.xml file 
    #
    # parameter
    #    line    line to replace
    #    params  parameter list (used for replacement)
    #    prefix  prefix of parameter (to be cut off for replacement)
    #
    # return 
    #    replaced line
    #
    #################################################################        
    def replaceTags(self, line, params, prefix):
        
        if line.find("@") > -1:
            tags = line.split("@")
            i = 1
            while i < len(tags):
                if params.has_key(prefix + "." + tags[i]):
                    paramName = "%s.%s" % (prefix, tags[i])
                    if verbose:
                        print("Replacing tag @%s@ by %s" % (tags[i], params[paramName]))
                    line = line.replace("@%s@" % tags[i], params[paramName])
                else:
                    if verbose:
                        print("Replacing tag @%s@ by empty string" %(tags[i]))
                    line = line.replace("@%s@" % tags[i], "")
                i += 2
        return line               
        
        
    #################################################################
    # start
    #
    # start data hydra service
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def start(self):

        # for the moment we do a restart if service is already running
        self.stop()   

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Starting the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
        
        # activate rgma servicetool (advanced)
        self.rgma_servicetool_activate = params['rgma.servicetool.activate']
        if (self.rgma_servicetool_activate != "true") and (self.rgma_servicetool_activate != "false"):
            glib.printFailedMessage()
            glib.printErrorMessage("Incorrect parameter 'rgma.servicetool.activate' given.")
            glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.rgma_servicetool_activate)
            return 1            
        
        #-------------------------------------------------------------------
        # Start MySQL
        #-------------------------------------------------------------------

        errorcode = mysql.start()
        if errorcode:
            return 1

        #-------------------------------------------------------------------
        # Start Tomcat
        #-------------------------------------------------------------------
            
        # for the moment we do a restart if service is already running
        #errorcode = tomcat.conditionalStart() only exists for secure version
        errorcode = tomcat.start()
        if errorcode:
            print "Starting %s service                             " % self.friendly_name,            
            glib.printFailedMessage()
            return 1
            
        print "Starting %s service                              " % self.friendly_name,            
        glib.printOkMessage()
        
        # give tomcat time to get up to full speed :)
        time.sleep(10)
        
        #--------------------------------------------------------
        # Start Servicetool
        #--------------------------------------------------------        
        print
        if self.rgma_servicetool_activate=="true":
            if rgmaServicetool.start():
                return 1

        return 0


    #################################################################
    # stop
    #
    # stop hydra service
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def stop(self):
        
        # activate rgma servicetool (advanced)
        self.rgma_servicetool_activate = params['rgma.servicetool.activate']
        if (self.rgma_servicetool_activate != "true") and (self.rgma_servicetool_activate != "false"):
            glib.printFailedMessage()
            glib.printErrorMessage("Incorrect parameter 'rgma.servicetool.activate' given.")
            glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.rgma_servicetool_activate)
            return 1            
        
        #--------------------------------------------------------        
        # Stop the servicetool
        #--------------------------------------------------------                
        if self.rgma_servicetool_activate=="true":
            if rgmaServicetool.stop():
                return 1        
        print
        
        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Stopping the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
            
        # ------------------------------------------------------------
        # stop tomcat
        # ------------------------------------------------------------
        
        errorcode = tomcat.stop()
        print "Stopping %s service                              " % self.friendly_name,
        if errorcode:
            glib.printFailedMessage()
            print
            return 1
        
        glib.printOkMessage()
        print
            
        return 0


    #################################################################
    # status
    #
    # get status of hydra service
    #
    # return  0  ok
    #        -1 error
    #         1  one/more services are not running
    #         2  all services are not running
    #
    #################################################################
    def status(self):

        errorcode = 0
        count = 0
        
        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Checking status of %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
         
        # activate rgma servicetool (advanced)
        self.rgma_servicetool_activate = params['rgma.servicetool.activate']
        if (self.rgma_servicetool_activate != "true") and (self.rgma_servicetool_activate != "false"):
            glib.printFailedMessage()
            glib.printErrorMessage("Incorrect parameter 'rgma.servicetool.activate' given.")
            glib.printErrorMessage("Must be 'true|false'. (Given parameter is '%s')" % self.rgma_servicetool_activate)
            return 1            

        hostcert_file = params['host.certificate.file']
        hostkey_file  = params['host.key.file']
        
        #--------------------------------------------------------
        # loop over VO's to get status
        #--------------------------------------------------------
        instances = glib.getInstances('hydra')

        if len(instances) > 0:

            for instance in instances:

                # extract vo name from 'default:voName' string
                voNameArray = instance.split(":")
                default_vo_name = voNameArray[1]
         
                try:
                    instanceParams = glib.getInstanceParams(instance)
                except ValueError, msg:
                    glib.printFailedMessage()
                    glib.printErrorMessage("%s, Cannot continue" % msg)
                    return -1
                except LookupError, msg:
                    glib.printFailedMessage()
                    glib.printErrorMessage(msg)
                    return -1

                # vo name - if not present: assign name of instance (user)
                if instanceParams.has_key('vo.name'):
                    self.vo_name = instanceParams['vo.name']
                else:
                    self.vo_name = default_vo_name
        
                status = glib.checkHTTPSconnection("localhost", 
                    "/%s/glite-data-hydra-service/" % self.vo_name,
                    8443, hostkey_file, hostcert_file)

                count = count +1
                if status[0]:
                    print "VO '%s' is not running." % self.vo_name
                    errorcode = errorcode + 1
                else:
                    print "VO '%s' is running." % self.vo_name
                    
        #--------------------------------------------------------
        # status of R-GMA servicetool
        #--------------------------------------------------------
        print
        
        if self.rgma_servicetool_activate=="true":
            count = count +1
            if rgmaServicetool.status():
                errorcode = errorcode + 1

        if errorcode == 0:
            return 0
        else:
            if errorcode == count:
                # nothing is running
                return 2
            else:
                # some services are not running
                return 1       
        
    
    #################################################################
    # showConfiguration
    #
    # print major configuration
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def showConfiguration(self):
        
        self.copyright()
        self.showVersion()
        glib.print_params(params)
        
        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "             Main settings of %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
        
        print "  HYDRA SERVER"
        print "     --------------------------------------------------------------"
        if params.has_key('hydra.mysql.admin.password'):
            print "     \033[34m[MySQL admin password]\x1b[0m          %s " % params['hydra.mysql.admin.password']
        else:
            glib.printErrorMessage("Parameter 'hydra.mysql.admin.password' not found.")
 
        if params.has_key('allow.unsecure.port'):
            print "     \033[34m[Unsecure port enabled]\x1b[0m         %s " % params['allow.unsecure.port']
        else:
            glib.printErrorMessage("Parameter 'hydra.mysql.admin.password' not found.")
 
 
        print
        print "  HYDRA VO settings"
        print "     --------------------------------------------------------------"

        # loop over VO's
        instances = glib.getInstances('hydra')

        if len(instances) > 0:
            for instance in instances:
                if self.showVoConfiguration(instance):
                    return

        print
        rgmaServicetool.showServices()
        return 0

        
    #################################################################
    # showVoConfiguration
    #
    # print major configuration for one VO
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def showVoConfiguration(self, name):

        # extract vo name from 'default:voName' string
        vo_name_array = name.split(":")
        default_vo_name = vo_name_array[1]
         
        try:
            iParams = glib.getInstanceParams(name)
        except ValueError, msg:
            glib.printFailedMessage()
            glib.printErrorMessage("%s, Cannot continue" % msg)
            return 1
        except LookupError, msg:
            glib.printFailedMessage()
            glib.printErrorMessage(msg)
            return 1

        #--------------------------------------------------------
        # hydra configuration values
        #--------------------------------------------------------

        # vo name - if not present: assign name of instance
        if iParams.has_key('vo.name'):
            vo_name = iParams['vo.name']
        else:
            vo_name = default_vo_name

        print
        print "     \033[34m[VO]\x1b[0m %s " % vo_name
        print "         =============================================================="

        # db name
        if iParams.has_key('hydra.DBNAME'):
            print "         \033[34m[DB name]\x1b[0m         %s " % iParams['hydra.DBNAME']
        else:
            glib.printErrorMessage("Parameter 'hydra.DBNAME' not found.")
        
        # db user
        if iParams.has_key('hydra.DBUSER'):
            print "         \033[34m[DB user]\x1b[0m         %s " % iParams['hydra.DBUSER']
        else:
            glib.printErrorMessage("Parameter 'hydra.DBUSER' not found.")
        
        # db password
        if iParams.has_key('hydra.DBPASSWORD'):
            print "         \033[34m[DB password]\x1b[0m     %s " % iParams['hydra.DBPASSWORD']
        else:
            glib.printErrorMessage("Parameter 'hydra.DBUSER' not found.")
        
        # db url
        if iParams.has_key('hydra.DBURL'):
            print "         \033[34m[DB url]\x1b[0m          %s " % iParams['hydra.DBURL']
        else:
            glib.printErrorMessage("Parameter 'hydra.DBURL' not found.")
        
        basePath = "https://%s:8443" % os.environ['HOSTNAME']
                
        # endpoint
        if iParams.has_key('hydra.PATH'):
            print "         \033[34m[Endpoint]\x1b[0m        %s%s " % ( basePath, iParams['hydra.PATH'])                                  
        else:
            glib.printErrorMessage("Parameter 'hydra.PATH' not found.")
        
        
        return 0
       

#################################################################
# loadDefaultParameters
#
# load default parameters
#
# params params list of parameters
#
# return 0 ok
#        1 error
#
#################################################################
def loadDefaultParameters(params):
    
    # hydra default params    
    params['glite.installer.checkcerts']       = "true"
    params['allow.unsecure.port']              = "false"
    params['hydra.DBDRIVERCLASS']              = "org.gjt.mm.mysql.Driver"
    params['hydra.DBRESOURCENAME']             = "meta"
    params['hydra.DOCBASE']                    = "${GLITE_LOCATION}/share/java/glite-data-hydra-service.war"
    params['hydra.ATTRIBUTE_HELPER_CLASS']     = "org.glite.data.hydra.helpers.AttributeHelper"
    params['hydra.CATALOG_HELPER_CLASS']       = "org.glite.data.hydra.helpers.CatalogHelper"
    params['hydra.SCHEMA_HELPER_CLASS']        = "org.glite.data.hydra.helpers.SchemaHelper"
    params['hydra.AUTHORIZATION_HELPER_CLASS'] = "org.glite.data.hydra.helpers.AuthorizationHelper"
    params['hydra.schemaFile']                 = "${GLITE_LOCATION}/etc/glite-data-hydra-service/schema/mysql/mysql-schema.sql"
    params['set.mysql.root.password']          = "false"
    
    # rgma default values
    params['rgma.servicetool.activate']        = "true"
    params['rgma.user.name']                   = "rgma"
    params['rgma.user.group']                  = "rgma"
    
    return 0

    
#################################################################
# set_env
#
# setup environment
# 
#################################################################
def set_env():
    
    # gLite
    glib.export('GLITE_LOCATION');
    glib.export('GLITE_LOCATION_VAR');
    if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
        os.mkdir(os.environ['GLITE_LOCATION_VAR'],0755)
    glib.export('GLITE_LOCATION_LOG');
    if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
        os.mkdir(os.environ['GLITE_LOCATION_LOG'],0755)
    glib.export('GLITE_LOCATION_TMP');
    if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
        os.mkdir(os.environ['GLITE_LOCATION_TMP'],0755)

    glib.export('CATALINA_HOME')
    glib.export('JAVA_HOME')

    # bin and lib paths
    glib.addEnvPath("PATH","%s/externals/bin:%s/bin:%s/bin" % (os.environ['GLITE_LOCATION'],os.environ['GLITE_LOCATION'],os.environ['JAVA_HOME']))
    
    # Set environment
    glib.setUserEnv()    
    
    
#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # The script must be run as root
    if not os.geteuid()==0:
        print '"\nThis script must be run as root\n'
        sys.exit(1)

    # Get an instance of the library class
    glib = gLib()
    glib.verbose = 0
    verboseCLI = 0
    
    # ------------------------------------------------------------
    # Load parameters
    # ------------------------------------------------------------
    
    # Get an instance of the ConfigParams class
    params = ConfigParams()
    
    # load default parameters
    loadDefaultParameters(params)
    
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version', 'start', 'stop', 'status','configure', 'siteconfig=', 'verbose'])
        for o, a in opts:
            
            if o == "--verbose":
                verboseCLI = 1
                glib.verbose = 1
                
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
                
    except getopt.GetoptError:
        pass

    # load rgma common and rgma server configuration parameters
    configFiles = ["%s/../glite-rgma-common.cfg.xml" % glib.getScriptPath(),
                   "%s/../glite-rgma-servicetool.cfg.xml" % glib.getScriptPath(), 
                   "%s/../glite-hydra.cfg.xml" % glib.getScriptPath()]
        
    if glib.loadConfiguration(configFiles,params):
        glib.printErrorMessage("Problems while loading configuration file.")
        sys.exit(1)

    # ------------------------------------------------------------    
    # set verbosity
    # ------------------------------------------------------------    
    verbose = 0
    if verboseCLI:
        verbose = 1
    else:
        if params.has_key('glite.installer.verbose'):
            if params['glite.installer.verbose'] == "true":
                verbose = 1
    
    glib.verbose = verbose    
    
    # ------------------------------------------------------------
    # Set up the environment
    # ------------------------------------------------------------
    set_env()
          
    # ------------------------------------------------------------          
    # Instantiate the service classes
    # ------------------------------------------------------------
    service = glite_hydra(params)
    service.verbose = verbose

    # ------------------------------------------------------------
    # Instantiate tomcat
    # ------------------------------------------------------------
    tomcat = Tomcat5(params)
    tomcat.verbose = verbose
    
    # ------------------------------------------------------------
    # Instantiate MySQL
    # ------------------------------------------------------------
    mysql = Mysql()
    mysql.verbose = verbose
    
    # ------------------------------------------------------------    
    # Instantiate the rgma servicetool class
    # ------------------------------------------------------------    
    rgmaServicetool = gliteRgmaServicetool()
    rgmaServicetool.verbose = verbose         
       
    # ------------------------------------------------------------
    # Command line opts if any
    # ------------------------------------------------------------
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version', 'start', 'stop', 'status','configure', 'siteconfig=', 'verbose'])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    # ------------------------------------------------------------
    # Check cli options
    # ------------------------------------------------------------
    for o, a in opts:
        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)
            
        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)
            
        # check configuration
        if o in ("-c", "--checkconf"):            
            service.showConfiguration()
            sys.exit(0)
            
        if o == "--start":
            if service.start() == 0:
                print "The %s was successfully started\n" % service.friendly_name
                sys.exit(0)
            else:
                print "An error occurred while starting the %s service" \
                    % service.friendly_name
                sys.exit(1)
                
        if o == "--stop":
            if service.stop() == 0:
                print "The %s was successfully stopped\n" % service.friendly_name
                sys.exit(0)
            else:
                print "An error occurred while stopping the %s service" \
                    % service.friendly_name
                sys.exit(1)
                
        if o == "--status":
            errorcode = service.status()
            
            if errorcode == 0:
                print "\nThe %s service is fully running." % service.friendly_name
                sys.exit(0)
            elif errorcode == 1:
                print "\nSome services of the %s service are not running." % service.friendly_name
                sys.exit(1)
            elif errorcode == 2:
                print "\nThe %s service is not running." % service.friendly_name
                sys.exit(1)
            else:
                print "\nAn error occured while checking the status of the %s service." % service.friendly_name
                sys.exit(1)                
                
        if o == "--configure":
            if service.configure() == 0:
                print "The %s was configuration successfully completed\n" % service.friendly_name
                sys.exit(0)
            else:
                print "An error occurred while configuring the %s service" % service.friendly_name
                sys.exit(1)        

    # ------------------------------------------------------------
    # Print configuration parameters
    # ------------------------------------------------------------
    if verbose:
        glib.print_params(params)

    service.copyright()
    service.showVersion()
    service.banner()
    service.usage()
